@@ -13,16 +13,16 @@
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import timeutils
+import six
 
 from senlin.common import consts
 from senlin.common import exception
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
-from senlin.common import utils
 from senlin.engine import cluster_policy as cpm
+from senlin.engine import health_manager
 from senlin.engine import node as node_mod
 from senlin.objects import cluster as co
 from senlin.objects import cluster_policy as cpo
+from senlin.objects import node as no
 from senlin.policies import base as pcb
 from senlin.profiles import base as pfb
 
@@ -39,14 +39,6 @@ class Cluster(object):
     executed.
     """
 
-    STATUSES = (
-        INIT, ACTIVE, CREATING, UPDATING, RESIZING, DELETING,
-        CHECKING, RECOVERING, CRITICAL, ERROR, WARNING,
-    ) = (
-        'INIT', 'ACTIVE', 'CREATING', 'UPDATING', 'RESIZING', 'DELETING',
-        'CHECKING', 'RECOVERING', 'CRITICAL', 'ERROR', 'WARNING',
-    )
-
     def __init__(self, name, desired_capacity, profile_id,
                  context=None, **kwargs):
         '''Initialize a cluster object.
@@ -76,11 +68,12 @@ class Cluster(object):
         self.timeout = (kwargs.get('timeout') or
                         cfg.CONF.default_action_timeout)
 
-        self.status = kwargs.get('status', self.INIT)
+        self.status = kwargs.get('status', consts.CS_INIT)
         self.status_reason = kwargs.get('status_reason', 'Initializing')
         self.data = kwargs.get('data', {})
         self.metadata = kwargs.get('metadata') or {}
         self.dependents = kwargs.get('dependents') or {}
+        self.config = kwargs.get('config') or {}
 
         # rt is a dict for runtime data
         self.rt = {
@@ -100,13 +93,14 @@ class Cluster(object):
         bindings = cpo.ClusterPolicy.get_all(context, self.id)
         for b in bindings:
             # Detect policy type conflicts
-            policy = pcb.Policy.load(context, b.policy_id)
+            policy = pcb.Policy.load(context, b.policy_id, project_safe=False)
             policies.append(policy)
 
         self.rt = {
-            'profile': pfb.Profile.load(context, profile_id=self.profile_id,
+            'profile': pfb.Profile.load(context,
+                                        profile_id=self.profile_id,
                                         project_safe=False),
-            'nodes': node_mod.Node.load_all(context, cluster_id=self.id),
+            'nodes': no.Node.get_all_by_cluster(context, self.id),
             'policies': policies
         }
 
@@ -135,6 +129,7 @@ class Cluster(object):
             'meta_data': self.metadata,
             'data': self.data,
             'dependents': self.dependents,
+            'config': self.config,
         }
 
         timestamp = timeutils.utcnow(True)
@@ -174,6 +169,7 @@ class Cluster(object):
             'data': obj.data,
             'metadata': obj.metadata,
             'dependents': obj.dependents,
+            'config': obj.config,
         }
 
         return cls(obj.name, obj.desired_capacity, obj.profile_id,
@@ -203,36 +199,6 @@ class Cluster(object):
             cluster = cls._from_object(context, obj)
             yield cluster
 
-    def to_dict(self):
-        info = {
-            'id': self.id,
-            'name': self.name,
-            'profile_id': self.profile_id,
-            'user': self.user,
-            'project': self.project,
-            'domain': self.domain,
-            'init_at': utils.isotime(self.init_at),
-            'created_at': utils.isotime(self.created_at),
-            'updated_at': utils.isotime(self.updated_at),
-            'min_size': self.min_size,
-            'max_size': self.max_size,
-            'desired_capacity': self.desired_capacity,
-            'timeout': self.timeout,
-            'status': self.status,
-            'status_reason': self.status_reason,
-            'metadata': self.metadata,
-            'data': self.data,
-            'dependents': self.dependents,
-            'nodes': [node.id for node in self.rt['nodes']],
-            'policies': [policy.id for policy in self.rt['policies']],
-        }
-        if self.rt['profile']:
-            info['profile_name'] = self.rt['profile'].name
-        else:
-            info['profile_name'] = None
-
-        return info
-
     def set_status(self, context, status, reason=None, **kwargs):
         """Set status of the cluster.
 
@@ -246,11 +212,11 @@ class Cluster(object):
 
         values = {}
         now = timeutils.utcnow(True)
-        if status == self.ACTIVE and self.status == self.CREATING:
+        if status == consts.CS_ACTIVE and self.status == consts.CS_CREATING:
             self.created_at = now
             values['created_at'] = now
-        elif (status == self.ACTIVE and
-              self.status in (self.UPDATING, self.RESIZING)):
+        elif (status == consts.CS_ACTIVE and
+              self.status in (consts.CS_UPDATING, consts.CS_RESIZING)):
             self.updated_at = now
             values['updated_at'] = now
 
@@ -272,30 +238,33 @@ class Cluster(object):
         co.Cluster.update(context, self.id, values)
         return
 
-    def update_dependents(self, context, dependents):
-        """Update dependency information of cluster's property.
-
-        :param context: An instance of request context.
-        :param dependents: The dependency information.
-        """
-
-        values = {'dependents': dependents}
-        co.Cluster.update(context, self.id, values)
-
     def do_create(self, context, **kwargs):
         '''Additional logic at the beginning of cluster creation process.
 
         Set cluster status to CREATING.
         '''
-        if self.status != self.INIT:
-            LOG.error(_LE('Cluster is in status "%s"'), self.status)
+        if self.status != consts.CS_INIT:
+            LOG.error('Cluster is in status "%s"', self.status)
+            return False
+
+        self.set_status(context, consts.CS_CREATING, 'Creation in progress')
+        try:
+            pfb.Profile.create_cluster_object(context, self)
+        except exception.EResourceCreation as ex:
+            self.set_status(context, consts.CS_ERROR, six.text_type(ex))
             return False
 
-        self.set_status(context, self.CREATING, _('Creation in progress'))
         return True
 
     def do_delete(self, context, **kwargs):
         """Additional logic at the end of cluster deletion process."""
+        self.set_status(context, consts.CS_DELETING, 'Deletion in progress')
+
+        try:
+            pfb.Profile.delete_cluster_object(context, self)
+        except exception.EResourceDeletion as ex:
+            self.set_status(context, consts.CS_ERROR, six.text_type(ex))
+            return False
 
         co.Cluster.delete(context, self.id)
         return True
@@ -305,7 +274,7 @@ class Cluster(object):
 
         This method is intended to be called only from an action.
         '''
-        self.set_status(context, self.UPDATING, _('Update in progress'))
+        self.set_status(context, consts.CS_UPDATING, 'Update in progress')
         return True
 
     def do_check(self, context, **kwargs):
@@ -313,7 +282,7 @@ class Cluster(object):
 
         Set cluster status to CHECKING.
         """
-        self.set_status(context, self.CHECKING, _('Check in progress'))
+        self.set_status(context, consts.CS_CHECKING, 'Check in progress')
         return True
 
     def do_recover(self, context, **kwargs):
@@ -321,7 +290,17 @@ class Cluster(object):
 
         Set cluster status to RECOVERING.
         '''
-        self.set_status(context, self.RECOVERING, _('Recovery in progress'))
+        self.set_status(context, consts.CS_RECOVERING, 'Recovery in progress')
+        return True
+
+    def do_operation(self, context, **kwargs):
+        """Additional logic at the beginning of cluster recovering process.
+
+        Set cluster status to OPERATING.
+        """
+        operation = kwargs.get("operation", "unknown")
+        self.set_status(context, consts.CS_OPERATING,
+                        "Operation %s in progress" % operation)
         return True
 
     def attach_policy(self, ctx, policy_id, values):
@@ -341,21 +320,22 @@ class Cluster(object):
         for existing in self.rt['policies']:
             # Policy already attached
             if existing.id == policy_id:
-                return True, _('Policy already attached.')
+                return True, 'Policy already attached.'
 
             # Detect policy type conflicts
             if (existing.type == policy.type) and policy.singleton:
-                reason = _('Only one instance of policy type (%(ptype)s) can '
-                           'be attached to a cluster, but another instance '
-                           '(%(existing)s) is found attached to the cluster '
-                           '(%(cluster)s) already.'
-                           ) % {'ptype': policy.type,
-                                'existing': existing.id,
-                                'cluster': self.id}
+                reason = ("Only one instance of policy type (%(ptype)s) can "
+                          "be attached to a cluster, but another instance "
+                          "(%(existing)s) is found attached to the cluster "
+                          "(%(cluster)s) already."
+                          ) % {'ptype': policy.type,
+                               'existing': existing.id,
+                               'cluster': self.id}
                 return False, reason
 
         # invoke policy callback
-        res, data = policy.attach(self)
+        enabled = bool(values.get('enabled', True))
+        res, data = policy.attach(self, enabled=enabled)
         if not res:
             return False, data
 
@@ -371,7 +351,7 @@ class Cluster(object):
         # refresh cached runtime
         self.rt['policies'].append(policy)
 
-        return True, _('Policy attached.')
+        return True, 'Policy attached.'
 
     def update_policy(self, ctx, policy_id, **values):
         """Update a policy that is already attached to a cluster.
@@ -390,16 +370,23 @@ class Cluster(object):
                 found = True
                 break
         if not found:
-            return False, _('Policy not attached.')
+            return False, 'Policy not attached.'
 
         enabled = values.get('enabled', None)
         if enabled is None:
-            return True, _('No update is needed.')
+            return True, 'No update is needed.'
 
         params = {'enabled': bool(enabled)}
+        # disable health check if necessary
+        policy_type = existing.type.split('-')[0]
+        if policy_type == 'senlin.policy.health':
+            if enabled is True:
+                health_manager.enable(self.id)
+            else:
+                health_manager.disable(self.id)
 
         cpo.ClusterPolicy.update(ctx, self.id, policy_id, params)
-        return True, _('Policy updated.')
+        return True, 'Policy updated.'
 
     def detach_policy(self, ctx, policy_id):
         """Detach policy object from the cluster.
@@ -418,7 +405,7 @@ class Cluster(object):
                 found = existing
                 break
         if found is None:
-            return False, _('Policy not attached.')
+            return False, 'Policy not attached.'
 
         policy = pcb.Policy.load(ctx, policy_id)
         res, reason = policy.detach(self)
@@ -428,7 +415,7 @@ class Cluster(object):
         cpo.ClusterPolicy.delete(ctx, self.id, policy_id)
         self.rt['policies'].remove(found)
 
-        return True, _('Policy detached.')
+        return True, 'Policy detached.'
 
     @property
     def nodes(self):
@@ -450,6 +437,13 @@ class Cluster(object):
             if node.id == node_id:
                 self.rt['nodes'].remove(node)
 
+    def update_node(self, nodes):
+        """Update cluster runtime data
+
+        :param nodes: List of node objects
+        """
+        self.rt['nodes'] = nodes
+
     @property
     def policies(self):
         return self.rt['policies']
@@ -515,7 +509,6 @@ class Cluster(object):
         :param zone: Name of availability zone for filtering.
         :return: A list of nodes that reside in the specified AZ.
         """
-        # TODO(anyone): Improve this to do a forced refresh via get_details?
         result = []
         for node in self.nodes:
             placement = node.data.get('placement', {})
@@ -524,6 +517,21 @@ class Cluster(object):
                     result.append(node)
         return result
 
+    def health_check(self, ctx):
+        """Check physical resources status
+
+        :param ctx: The context to operate node object
+        """
+        # Note this procedure is a pure sequential operation,
+        # its not suitable for large scale clusters.
+
+        old_nodes = self.nodes
+        for node in old_nodes:
+            node.do_check(ctx)
+
+        nodes = node_mod.Node.load_all(ctx, cluster_id=self.id)
+        self.update_node([n for n in nodes])
+
     def eval_status(self, ctx, operation, **params):
         """Re-evaluate cluster's health status.
 
@@ -531,39 +539,38 @@ class Cluster(object):
         :param operation: The operation that triggers this status evaluation.
         :returns: ``None``.
         """
-        self.rt['nodes'] = node_mod.Node.load_all(ctx, cluster_id=self.id)
+        nodes = node_mod.Node.load_all(ctx, cluster_id=self.id)
+        self.rt['nodes'] = [n for n in nodes]
+
         active_count = 0
         for node in self.nodes:
             if node.status == 'ACTIVE':
                 active_count += 1
 
         # get provided desired_capacity/min_size/max_size
-        desired = params.get('desired_capacity', None)
-        desired = desired or self.desired_capacity
-        min_size = params.get('min_size', None)
-        min_size = min_size or self.min_size
-        max_size = params.get('max_size', None)
-        max_size = max_size or self.max_size
+        desired = params.get('desired_capacity', self.desired_capacity)
+        min_size = params.get('min_size', self.min_size)
+        max_size = params.get('max_size', self.max_size)
 
         values = params or {}
         if active_count < min_size:
-            status = self.ERROR
-            reason = _("%(o)s: number of active nodes is below min_size "
-                       "(%(n)d).") % {'o': operation, 'n': min_size}
+            status = consts.CS_ERROR
+            reason = ("%(o)s: number of active nodes is below min_size "
+                      "(%(n)d).") % {'o': operation, 'n': min_size}
         elif active_count < desired:
-            status = self.WARNING
-            reason = _("%(o)s: number of active nodes is below "
-                       "desired_capacity "
-                       "(%(n)d).") % {'o': operation, 'n': desired}
+            status = consts.CS_WARNING
+            reason = ("%(o)s: number of active nodes is below "
+                      "desired_capacity "
+                      "(%(n)d).") % {'o': operation, 'n': desired}
         elif max_size < 0 or active_count <= max_size:
-            status = self.ACTIVE
-            reason = _("%(o)s: number of active nodes is above "
-                       "desired_capacity "
-                       "(%(n)d).") % {'o': operation, 'n': desired}
+            status = consts.CS_ACTIVE
+            reason = ("%(o)s: number of active nodes is equal or above "
+                      "desired_capacity "
+                      "(%(n)d).") % {'o': operation, 'n': desired}
         else:
-            status = self.WARNING
-            reason = _("%(o)s: number of active nodes is above max_size "
-                       "(%(n)d).") % {'o': operation, 'n': max_size}
+            status = consts.CS_WARNING
+            reason = ("%(o)s: number of active nodes is above max_size "
+                      "(%(n)d).") % {'o': operation, 'n': max_size}
 
         values.update({'status': status, 'status_reason': reason})
         co.Cluster.update(ctx, self.id, values)
