@@ -14,19 +14,18 @@ import itertools
 
 from oslo_log import log as logging
 
-from senlin.common.i18n import _LI, _LW
 
 LOG = logging.getLogger(__name__)
 
 
 class PluginInfo(object):
-    '''Base mapping of plugin type to implementation.'''
+    """Base mapping of plugin type to implementation."""
 
     def __new__(cls, registry, name, plugin, **kwargs):
-        '''Create a new PluginInfo of the appropriate class.
+        """Create a new PluginInfo of the appropriate class.
 
         Placeholder for class hierarchy extensibility
-        '''
+        """
         return super(PluginInfo, cls).__new__(cls)
 
     def __init__(self, registry, name, plugin):
@@ -63,7 +62,7 @@ class PluginInfo(object):
 
 
 class Registry(object):
-    '''A registry for managing profile or policy classes.'''
+    """A registry for managing profile or policy classes."""
 
     def __init__(self, registry_name, global_registry=None):
         self.registry_name = registry_name
@@ -72,16 +71,16 @@ class Registry(object):
         self.global_registry = global_registry
 
     def _register_info(self, name, info):
-        '''place the new info in the correct location in the registry.
+        """place the new info in the correct location in the registry.
 
-        :param path: a string of plugin name.
+        :param name: a string of plugin name.
         :param info: reference to a PluginInfo data structure, deregister a
                      PluginInfo if specified as None.
-        '''
+        """
         registry = self._registry
         if info is None:
             # delete this entry.
-            msg = _LW("Removing %(item)s from registry")
+            msg = "Removing %(item)s from registry"
             LOG.warning(msg, {'item': name})
             registry.pop(name, None)
             return
@@ -94,10 +93,10 @@ class Registry(object):
                 'old': str(registry[name].plugin),
                 'new': str(info.plugin)
             }
-            LOG.warning(_LW('Changing %(name)s from %(old)s to %(new)s'),
+            LOG.warning('Changing %(name)s from %(old)s to %(new)s',
                         details)
         else:
-            msg = _LI('Registering %(name)s -> %(value)s')
+            msg = 'Registering %(name)s -> %(value)s'
             LOG.info(msg, {'name': name, 'value': str(info.plugin)})
 
         info.user_provided = not self.is_global
@@ -132,5 +131,13 @@ class Registry(object):
         return dict((k, v.plugin) for k, v in self._registry.items())
 
     def get_types(self):
-        '''Return a list of valid plugin types.'''
-        return [{'name': name} for name in self._registry.keys()]
+        """Return a list of valid plugin types."""
+        types_support = []
+        for tn, ts in self._registry.items():
+            name = tn.split('-')[0] if '-' in tn else tn
+            version = tn.split('-')[1] if '-' in tn else ''
+            support = ts.plugin.VERSIONS[version] if version != '' else ''
+            pi = {version: support}
+            types_support.append({'name': name, 'version': version,
+                                  'support_status': pi})
+        return types_support
